<?php

/**
 * @file
 * Contains CdekRequestManager class.
 */

/**
 * CDEK API request manager.
 */
class CdekRequestManager {

  /**
   * Request path for order pickup points list.
   */
  const ORDER_PICKUP_POINTS_LIST_PATH = 'pvzlist/v1/json';

  /**
   * Request path for delivery rate calculation.
   */
  const DELIVERY_RATE_CALCULATION_PATH = 'calculator/calculate_price_by_json.php';

  /**
   * CDEK API settings.
   *
   * @var array
   */
  protected $settings = array();

  /**
   * CdekRequestManager constructor.
   */
  public function __construct() {
    // Get the configuration values.
    $this->settings = Cdek::getSettings();
  }

  /**
   * Returns response from "order pickup points list" API request.
   *
   * @param array $params
   *   See Cdek::getOrderPickupPoints() for details.
   *
   * @return string|null
   *   Response body or NULL on failure.
   */
  public function sendOrderPickupPointsRequest(array $params = array()) {
    // Request options and URL.
    $url = $this->settings['integration_base_path'] . '/' . self::ORDER_PICKUP_POINTS_LIST_PATH;
    $url = url($url, array('absolute' => TRUE, 'query' => $params));
    $options['method'] = 'GET';
    $options['timeout'] = (float) $this->settings['request_timeout'];
    // Sending GET request.
    $response = drupal_http_request($url, $options);
    // Check the success of the API request.
    return $this->isSuccessfulRequest($response) ? (string) $response->data : NULL;
  }

  /**
   * Returns response from "calculate delivery rate" API request.
   *
   * @param array $params
   *   See Cdek::calculateDeliveryRate() for details.
   *
   * @return string|null
   *   Response body or NULL on failure.
   */
  public function sendDeliveryRateCalculationRequest(array $params = array()) {
    // Request options and URL.
    $url = $this->settings['base_path'] . '/' . self::DELIVERY_RATE_CALCULATION_PATH;
    $url = url($url, array('absolute' => TRUE));
    $options['headers'] = array('Content-Type' => 'application/json');
    $options['method'] = 'POST';
    $options['data'] = json_encode($params);
    $options['timeout'] = (float) $this->settings['request_timeout'];
    // Sending POST request.
    $response = drupal_http_request($url, $options);
    // Check the success of the API request.
    return $this->isSuccessfulRequest($response) ? (string) $response->data : NULL;
  }

  /**
   * Check the success of the API request.
   *
   * @param object $response
   *   Response of the API request.
   *
   * @return bool
   *   The request is successful (TRUE) or not (FALSE).
   */
  protected function isSuccessfulRequest($response) {
    return empty($response->error) && $response->code >= 200 && $response->code < 300;
  }

}
